package org.jbpm.examples.junit;



import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.process.workitem.wsht.WSHumanTaskHandler;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.mina.MinaTaskClientConnector;
import org.jbpm.task.service.mina.MinaTaskClientHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskOperationResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskSummaryResponseHandler;

/**
 * This is a sample file to launch a process.
 */
public class TestJbpm5 {

	public static final void main(String[] args) {
		TestJbpm5 t = new TestJbpm5();
		t.manageMentHumanTask("krisv");
	}
	private  void manageMentHumanTask(String userId){
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(ksession, "test", 1000);
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task", new WSHumanTaskHandler());
			// start a new process instance
			Map<String, Object> params = new HashMap<String, Object>();
			params.put("employee", userId);
			params.put("reason", "Yearly performance evaluation");
			ProcessInstance process =ksession.startProcess("com.sample.evaluation", params);

			SystemEventListenerFactory.setSystemEventListener(new SystemEventListener());
			TaskClient taskClient = new TaskClient(new MinaTaskClientConnector("MinaConnector",
				new MinaTaskClientHandler(SystemEventListenerFactory.getSystemEventListener())));
			taskClient.connect("127.0.0.1", 9123);
			Thread.sleep(1000);
			
			// "krisv" executes his own performance evaluation
			BlockingTaskSummaryResponseHandler taskSummaryHandler = new BlockingTaskSummaryResponseHandler();
			taskClient.getTasksAssignedAsPotentialOwner("krisv", "en-UK", taskSummaryHandler);
			TaskSummary task1 = taskSummaryHandler.getResults().get(0);
			System.out.println("Krisv executing task " + task1.getName() + "(" + task1.getId() + ": " + task1.getDescription() + ")"+"status:"+task1.getStatus());
			
			BlockingTaskOperationResponseHandler taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.start(task1.getId(), "krisv", taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.complete(task1.getId(), "krisv", null, taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			Thread.sleep(1000);
			
			List<String> groups = new ArrayList<String>();
			groups.add("HR");
			// "john", part of the "PM" group, executes a performance evaluation
			taskSummaryHandler = new BlockingTaskSummaryResponseHandler();
			taskClient.getTasksAssignedAsPotentialOwner("mary", groups, "en-UK", taskSummaryHandler);
			TaskSummary task2 = taskSummaryHandler.getResults().get(0);
			System.out.println("mary executing task " + task2.getName() + "(" + task2.getId() + ": " + task2.getDescription() + ")"+"status:"+task1.getStatus());
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.claim(task2.getId(), "mary", groups, taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.start(task2.getId(), "mary", taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.complete(task2.getId(), "mary", null, taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			Thread.sleep(1000);	
			
			taskSummaryHandler = new BlockingTaskSummaryResponseHandler();
			taskClient.getTasksOwned(userId, "en-UK", taskSummaryHandler);
			//taskClient.getTasksAssignedAsPotentialOwner("krisv", "en-UK", taskSummaryHandler);
			task1 = taskSummaryHandler.getResults().get(0);
			System.out.println("Krisv executing task " + task1.getName() + "(" + task1.getId() + ": " + task1.getDescription() + ")"+"status:"+task1.getStatus());
			
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.start(task1.getId(), "krisv", taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			taskOperationHandler = new BlockingTaskOperationResponseHandler();
			taskClient.complete(task1.getId(), "krisv", null, taskOperationHandler);
			taskOperationHandler.waitTillDone(1000);
			Thread.sleep(1000);

			logger.close();
			
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}		

	
	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("TestJbpm.bpmn"), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}

	
	private static class SystemEventListener implements org.drools.SystemEventListener {
		public void debug(String arg0) {
		}
		public void debug(String arg0, Object arg1) {
		}
		public void exception(Throwable arg0) {
		}
		public void exception(String arg0, Throwable arg1) {
		}
		public void info(String arg0) {
		}
		public void info(String arg0, Object arg1) {
		}
		public void warning(String arg0) {
		}
		public void warning(String arg0, Object arg1) {
		}
	}

}
